import React, { createContext, useState, useContext, useEffect, useMemo, isValidElement } from 'react';
import { Dimensions, Platform, StatusBar, StyleSheet, Pressable, Text, View, useWindowDimensions, SafeAreaView, ScrollView } from 'react-native';

function isIphoneX() {
    const dimen = Dimensions.get('window');
    return (
        Platform.OS === 'ios' &&
        !Platform.isPad &&
        !Platform.isTVOS &&
        ((dimen.height === 780 || dimen.width === 780)
          || (dimen.height === 812 || dimen.width === 812)
          || (dimen.height === 844 || dimen.width === 844)
          || (dimen.height === 896 || dimen.width === 896)
          || (dimen.height === 926 || dimen.width === 926))
    );
}

// guideline height for standard 5" device screen is 680
function RFValue(fontSize, standardScreenHeight = 680) {
  const { height, width } = Dimensions.get("window");
  const standardLength = width > height ? width : height;
  const offset =
    width > height ? 0 : Platform.OS === "ios" ? 78 : StatusBar.currentHeight; // iPhone X style SafeAreaView size in portrait

  const deviceHeight =
    isIphoneX() || Platform.OS === "android"
      ? standardLength - offset
      : standardLength;

  const heightPercent = (fontSize * deviceHeight) / standardScreenHeight;
  return Math.round(heightPercent);
}

const ProgressButton = ({ title, onPress, style, titleStyle, disabled, children, }) => {
    return (React.createElement(Pressable, { disabled: disabled, onPress: onPress, style: ({ pressed }) => [
            pressed && styles$3.pressed,
            styles$3.buttonStyle,
            {
                ...style,
                backgroundColor: disabled
                    ? "#cdcdcd"
                    : style?.backgroundColor ?? "black",
                minWidth: 100,
                borderRadius: 8,
            },
        ] },
        children,
        title && (React.createElement(Text, { style: [styles$3.title, titleStyle], numberOfLines: 1, minimumFontScale: 0.8, adjustsFontSizeToFit: true }, title))));
};
const styles$3 = StyleSheet.create({
    buttonStyle: {
        justifyContent: "center",
        alignItems: "center",
        height: "60%",
    },
    title: {
        textAlign: "center",
        color: "white",
        fontSize: RFValue(16),
        fontWeight: "bold",
        paddingHorizontal: "4%",
    },
    pressed: { opacity: 0.3 },
});

const ProcessFooter = ({ removeButtonRow, hidePreviousButton, previousButtonStyle, previousButtonText, previousButtonDisabled, onPreviousStep, previousButtonTextStyle, hideNextButton, nextButtonStyle, nextButtonTitle, onNextStep, nextButtonTextStyle, nextButtonDisabled, footerComponent, }) => {
    return removeButtonRow ? (React.createElement(View, { style: styles$2.footerContainer },
        React.createElement(View, { style: styles$2.footerNestedContainer },
            !footerComponent && (React.createElement(View, { style: styles$2.footerBtnContainer },
                hidePreviousButton ? (React.createElement(ProgressButton, { style: previousButtonStyle, title: previousButtonText ? previousButtonText : "", disabled: previousButtonDisabled, onPress: onPreviousStep, titleStyle: previousButtonTextStyle })) : (React.createElement(View, null)),
                React.createElement(View, null, hideNextButton && (React.createElement(ProgressButton, { style: nextButtonStyle, title: nextButtonTitle, onPress: onNextStep, titleStyle: nextButtonTextStyle, disabled: nextButtonDisabled }))))),
            footerComponent && (React.createElement(View, { style: styles$2.footerComponentContainer }, footerComponent))))) : null;
};
const styles$2 = StyleSheet.create({
    footerContainer: {
        overflow: "hidden",
        backgroundColor: "transparent",
        paddingTop: "2%",
        height: "10%",
        minHeight: 70,
        maxHeight: 100,
    },
    footerNestedContainer: {
        flex: 1,
        shadowColor: "#000",
        shadowOffset: {
            width: 0,
            height: 3,
        },
        shadowOpacity: 0.27,
        shadowRadius: 4.65,
        elevation: 6,
    },
    footerComponentContainer: {
        flex: 1,
        backgroundColor: "white",
    },
    footerBtnContainer: {
        flexDirection: "row",
        justifyContent: "space-between",
        alignItems: "center",
        paddingHorizontal: "10%",
        flex: 1,
        backgroundColor: "white",
    },
});

const ProcessContext = createContext({
    totalSteps: 0,
    numberOfSteps: (_) => { },
    activeStep: 0,
    currentStep: (_) => { }
});
const ProcessContainer = ({ children, initialNumberOfSteps = 1 }) => {
    const [activeStep, setActiveStep] = useState(0);
    const [totalSteps, setTotalSteps] = useState(initialNumberOfSteps);
    const numberOfSteps = (totalStepsNumber) => {
        if (totalStepsNumber > 0) {
            setTotalSteps(totalStepsNumber);
            if (activeStep >= totalStepsNumber) {
                setActiveStep(totalStepsNumber - 1);
            }
        }
        else {
            console.error('Please Enter Positive Number');
        }
    };
    const currentStep = (currentStepIndex) => {
        if (currentStepIndex >= 0 && currentStepIndex < totalSteps) {
            setActiveStep(currentStepIndex);
        }
        else {
            setActiveStep(0);
        }
    };
    const value = {
        totalSteps,
        numberOfSteps,
        activeStep,
        currentStep
    };
    return React.createElement(ProcessContext.Provider, { value: value }, children);
};

/* eslint-disable react-native/no-inline-styles */
const StepIcon = ({ label, labelStyle, stepNumber, isActiveStep, isCompletedStep, completedStepIconColor = "#4BB543", activeStepIconColor = "#87CEEB", activeStepNumColor = "black", disabledStepNumColor = "white", completedCheckColor = "white", showLabelAboveSteps = false, }) => {
    const { width } = useWindowDimensions();
    const { totalSteps } = useContext(ProcessContext);
    let stylesConfig = {};
    if (isActiveStep) {
        stylesConfig = {
            circleStyle: {
                backgroundColor: activeStepIconColor,
                shadowColor: activeStepIconColor,
                shadowOffset: {
                    width: 0,
                    height: 0,
                },
                shadowOpacity: 0.5,
                shadowRadius: 4.65,
                elevation: 6,
            },
            stepNum: {
                color: activeStepNumColor,
                fontSize: RFValue(12),
            },
        };
    }
    else if (isCompletedStep) {
        stylesConfig = {
            circleStyle: {
                backgroundColor: completedStepIconColor,
            },
        };
    }
    else {
        stylesConfig = {
            circleStyle: {
                backgroundColor: "lightgray",
            },
            stepNum: {
                color: disabledStepNumColor,
                fontSize: RFValue(12),
            },
        };
    }
    return (React.createElement(View, { style: styles$1.stepIconContainer },
        React.createElement(View, { style: styles$1.stepIconInnerContainer },
            showLabelAboveSteps && (React.createElement(View, { style: [
                    styles$1.labelContainer,
                    {
                        paddingTop: "2%",
                        alignItems: "center",
                        flex: 1,
                        height: (width * 10) / 100,
                        maxHeight: 50,
                    },
                ] },
                React.createElement(Text, { style: [
                        styles$1.label,
                        labelStyle,
                        {
                            width: totalSteps > 8 ? width / 7 - 4 : width / totalSteps - 4,
                            color: isActiveStep
                                ? labelStyle?.color
                                    ? labelStyle?.color
                                    : "black"
                                : "transparent",
                        },
                    ], numberOfLines: 2, minimumFontScale: 0.9, adjustsFontSizeToFit: Platform.OS === "ios" ? true : false }, label))),
            React.createElement(View, { style: [
                    styles$1.stepIconCircleContainer,
                    showLabelAboveSteps && {
                        paddingBottom: "4%",
                    },
                ] },
                React.createElement(View, { style: [
                        {
                            width: (width * 10) / 100,
                            height: (width * 10) / 100,
                            borderRadius: (width * 10) / 100 / 2,
                            justifyContent: "center",
                            alignItems: "center",
                            maxHeight: 60,
                            maxWidth: 60,
                        },
                        stylesConfig.circleStyle,
                    ] },
                    React.createElement(Text, { style: styles$1.circleText }, isCompletedStep ? (React.createElement(Text, { style: { color: completedCheckColor } }, "\u2713")) : (React.createElement(Text, { style: stylesConfig?.stepNum }, stepNumber))))),
            !showLabelAboveSteps && (React.createElement(View, { style: [
                    styles$1.labelContainer,
                    {
                        alignItems: "center",
                        flex: 1,
                        height: (width * 10) / 100,
                        maxHeight: 50,
                    },
                ] },
                React.createElement(Text, { style: [
                        styles$1.label,
                        labelStyle,
                        {
                            width: totalSteps > 8 ? width / 7 - 4 : width / totalSteps - 4,
                            color: isActiveStep
                                ? labelStyle?.color
                                    ? labelStyle?.color
                                    : "black"
                                : "transparent",
                        },
                    ], numberOfLines: 2, minimumFontScale: 0.9, adjustsFontSizeToFit: Platform.OS === "ios" ? true : false }, label))))));
};
const styles$1 = StyleSheet.create({
    stepIconContainer: {
        width: "100%",
        height: "100%",
        justifyContent: "center",
        alignItems: "center",
    },
    stepIconInnerContainer: {
        flex: 1,
        width: "100%",
        height: "100%",
        justifyContent: "center",
        alignItems: "center",
    },
    stepIconCircleContainer: {
        width: "100%",
        justifyContent: "center",
        alignItems: "center",
        maxHeight: 60,
        maxWidth: 60,
    },
    labelContainer: {
        width: "100%",
    },
    label: {
        textAlign: "center",
        flex: 1,
        fontWeight: "bold",
        fontSize: RFValue(12),
    },
    circleText: {
        alignSelf: "center",
        fontWeight: "bold",
    },
});

/* eslint-disable react-native/no-inline-styles */
const ProcessFlow = ({ 
// isComplete not perfectly working now
// isComplete = false,
children, onChangeStep, 
// Buttons Config
onNext, onPrevious, onSubmit, nextButtonStyle, nextButtonText = "Next", nextButtonTextStyle, previousButtonStyle, previousButtonText = "Previous", previousButtonTextStyle, finishButtonText = "Submit", nextButtonDisabled = false, previousButtonDisabled = false, removeButtonRow = false, footerComponent, labelStyle, showLabelAboveSteps = false, ...props }) => {
    const { activeStep, currentStep, numberOfSteps, totalSteps } = useContext(ProcessContext);
    const { width } = useWindowDimensions();
    const [stepIconWidthOffSet, setStepIconWidthOffSet] = useState([]);
    useEffect(() => {
        if (activeStep >= 0 && activeStep < totalSteps) {
            if (onChangeStep) {
                onChangeStep(activeStep, activeStep + 1, activeStep - 1);
            }
        }
    }, [onChangeStep, activeStep, totalSteps]);
    // useEffect(() => {
    //   if (isComplete) {
    //     currentStep(totalSteps - 1);
    //   }
    // }, [currentStep, isComplete, totalSteps]);
    const renderChildren = React.useMemo(() => {
        const tmp = React.Children.map(children, (ele) => {
            if (ele && React.isValidElement(ele) && !ele?.props?.hide) {
                return React.cloneElement(ele);
            }
        });
        numberOfSteps(tmp?.length);
        return tmp;
    }, [children, numberOfSteps]);
    const renderStepIcons = () => {
        let step = [];
        let i = 0;
        while (i !== totalSteps) {
            const isCompletedStep = i < activeStep;
            const isActiveStep = i === activeStep;
            step.push(React.createElement(View, { key: i, style: { flex: 1 }, onLayout: ({ nativeEvent: offSet }) => {
                    setStepIconWidthOffSet((s) => {
                        return [...s, offSet].slice(-totalSteps).sort((a, b) => {
                            return a.layout.x - b.layout.x;
                        });
                    });
                } },
                React.createElement(StepIcon, { ...{ ...props, showLabelAboveSteps }, stepNumber: i + 1, label: renderChildren[i]?.props?.label ?? "", labelStyle: labelStyle
                        ? labelStyle
                        : renderChildren[i]?.props?.labelStyle
                            ? renderChildren[i]?.props?.labelStyle
                            : undefined, isCompletedStep: isCompletedStep, isActiveStep: isActiveStep })));
            i++;
        }
        return step;
    };
    const onNextStep = async () => {
        const isThereError = onNext && !!(await onNext(activeStep + 1));
        const isError = renderChildren[activeStep]?.props?.onNext &&
            !!(await renderChildren[activeStep]?.props?.onNext(activeStep + 1));
        // Return out of method before moving to next step if errors exist.
        if (isThereError || isError) {
            return;
        }
        currentStep(activeStep + 1);
    };
    const onPreviousStep = async () => {
        const isThereError = onPrevious && !!(await onPrevious(activeStep + 1));
        const isError = renderChildren[activeStep]?.props?.onPrevious &&
            !!(await renderChildren[activeStep]?.props?.onPrevious(activeStep + 1));
        if (isThereError || isError) {
            return;
        }
        // Changes active index and calls previous function passed by parent
        if (activeStep >= 0) {
            currentStep(activeStep - 1);
        }
    };
    const onSubmitHandler = async () => {
        onSubmit && onSubmit();
    };
    const indicatorTop = useMemo(() => showLabelAboveSteps
        ? (width * 10) / 100 >= 50
            ? 77
            : (width * 27) / 100 / 2
        : (width * 10) / 100 >= 50
            ? 30
            : (width * 10) / 100 / 2, [width, showLabelAboveSteps]);
    return (React.createElement(SafeAreaView, { style: styles.container },
        React.createElement(View, { style: {
                marginVertical: 10,
                maxHeight: 115,
            } },
            totalSteps < 9 && (React.createElement(View, { style: [
                    styles.stepIcons,
                    {
                        maxHeight: 115,
                        borderBottomWidth: 1,
                        borderBottomColor: "lightgray",
                        height: (width * 20) / 100,
                        width: width,
                    },
                ] },
                renderStepIcons(),
                React.createElement(View, { style: [
                        styles.stepIndicatorOuter,
                        {
                            top: indicatorTop,
                            width: stepIconWidthOffSet[totalSteps - 1]?.layout?.x,
                        },
                    ] },
                    React.createElement(View, { style: [
                            styles.stepIndicatorInner,
                            {
                                width: stepIconWidthOffSet[activeStep]?.layout?.x,
                                borderWidth: 1.5,
                            },
                        ] })))),
            totalSteps >= 9 && (React.createElement(ScrollView, { horizontal: true, bounces: false, contentContainerStyle: [
                    {
                        flexDirection: "row",
                        justifyContent: "center",
                        alignItems: "center",
                        alignSelf: "center",
                    },
                ], style: [
                    {
                        maxHeight: 115,
                        borderBottomWidth: 1,
                        borderBottomColor: "lightgray",
                        height: (width * 20) / 100,
                        width: width,
                        backgroundColor: "white",
                    },
                ] },
                renderStepIcons(),
                React.createElement(View, { style: [
                        styles.stepIndicatorOuter,
                        {
                            // Scrollable
                            top: indicatorTop,
                            width: stepIconWidthOffSet[totalSteps - 1]?.layout?.x,
                        },
                    ] },
                    React.createElement(View, { style: [
                            styles.stepIndicatorInner,
                            {
                                width: stepIconWidthOffSet[activeStep]?.layout?.x,
                                borderWidth: 1.5,
                            },
                        ] }))))),
        React.createElement(View, { style: styles.bodyContainer }, isValidElement(renderChildren[activeStep]) &&
            React.cloneElement(renderChildren[activeStep])),
        React.createElement(ProcessFooter, { footerComponent: footerComponent ||
                renderChildren[activeStep]?.props?.footerComponent ? (React.createElement(React.Fragment, null,
                footerComponent && footerComponent,
                renderChildren[activeStep]?.props?.footerComponent &&
                    renderChildren[activeStep]?.props?.footerComponent)) : undefined, removeButtonRow: !removeButtonRow &&
                !renderChildren[activeStep]?.props?.removeButtonRow, hidePreviousButton: !renderChildren[activeStep]?.props?.hidePreviousButton &&
                (!(activeStep === 0) ||
                    renderChildren[activeStep]?.props?.showFirstStepPreviousButton), previousButtonStyle: previousButtonStyle, previousButtonText: previousButtonText
                ? renderChildren[activeStep]?.props?.previousButtonText
                    ? renderChildren[activeStep]?.props?.previousButtonText
                    : previousButtonText
                : "", previousButtonDisabled: previousButtonDisabled, onPreviousStep: onPreviousStep, previousButtonTextStyle: previousButtonTextStyle, hideNextButton: !renderChildren[activeStep]?.props?.hideNextButton, nextButtonStyle: nextButtonStyle, nextButtonTitle: activeStep === totalSteps - 1
                ? finishButtonText
                : renderChildren[activeStep]?.props?.nextButtonText
                    ? renderChildren[activeStep]?.props?.nextButtonText
                    : nextButtonText, onNextStep: activeStep === totalSteps - 1 ? onSubmitHandler : onNextStep, nextButtonTextStyle: nextButtonTextStyle, nextButtonDisabled: nextButtonDisabled })));
};
const styles = StyleSheet.create({
    container: {
        flex: 1,
    },
    stepIndicatorOuter: {
        zIndex: -1,
        backgroundColor: "lightgray",
        position: "absolute",
        alignSelf: "center",
    },
    stepIndicatorInner: {
        borderColor: "#4BB543",
    },
    bodyContainer: {
        flex: 1,
    },
    stepIcons: {
        flexDirection: "row",
        justifyContent: "center",
        alignItems: "center",
        alignSelf: "center",
        backgroundColor: "white",
    },
});

const Step = ({ children }) => {
    return React.createElement(React.Fragment, null, children);
};

const createProcess = () => {
    return {
        ProcessFlow,
        Step,
    };
};

const useProcess = () => {
    const process = useContext(ProcessContext);
    return process;
};

export { ProcessContainer, ProcessFlow, ProcessFooter, ProgressButton, Step, StepIcon, createProcess, useProcess };
//# sourceMappingURL=index.js.map
